<?php

/**
 * Created by PhpStorm.
 * User: Samuele Carpene
 * Web: http://www.samuelecarpene.com
 * Date: 31/08/2016
 * Time: 14:33
 */
class Mbe_Shipping_Model_Carrier
    extends Mage_Shipping_Model_Carrier_Abstract
    implements Mage_Shipping_Model_Carrier_Interface
{

    const SHIPMENT_CONFIGURATION_MODE_ONE_SHIPMENT_PER_ITEM = 1;
    const SHIPMENT_CONFIGURATION_MODE_ONE_SHIPMENT_PER_SHOPPING_CART_SINGLE_PARCEL = 2;
    const SHIPMENT_CONFIGURATION_MODE_ONE_SHIPMENT_PER_SHOPPING_CART_MULTI_PARCEL = 3;

    const HANDLING_TYPE_PER_SHIPMENT = "S";
    const HANDLING_TYPE_PER_PARCEL = "P";


    /**
     * Carrier's code, as defined in parent class
     *
     * @var string
     */
    protected $_code = 'mbe_shipping';

    /** @var $logger Mbe_Shipping_Helper_Logger */
    protected $logger = null;
    /** @var $helper Mbe_Shipping_Helper_Data */
    protected $helper = null;


    public function _construct()
    {
        $this->helper = Mage::helper('mbe_shipping');
        $this->logger = Mage::helper('mbe_shipping/logger');
    }


    public function getConfigData($field)
    {
        if (empty($this->_code))
        {
            return false;
        }
        $path = 'carriers/' . $this->_code . '/' . $field;
        return Mage::getStoreConfig($path, $this->getStore());
    }

    public function getRequestWeight(Mage_Shipping_Model_Rate_Request $request)
    {
        /*
        $result = 0;
        foreach ($request->getAllItems() as $item) {
            $result += $item->getWeight();
        }
        return $result;
        */
        return $request->getPackageWeight();
    }

    /**
     * Returns available shipping rates for Inchoo Shipping carrier
     *
     * @param Mage_Shipping_Model_Rate_Request $request
     * @return Mage_Shipping_Model_Rate_Result
     */
    public function collectRates(Mage_Shipping_Model_Rate_Request $request)
    {

        $this->logger->log('collectRates');


        //TODO: check countries
        /**
         * @var $helper Mbe_Shipping_Helper_Data
         * @var $request Mage_Shipping_Model_Rate_Request
         */
        $helper = Mage::helper('mbe_shipping');


        if (!$helper->isEnabled($this->getStoreId()))
        {
            $this->logger->log('module disabled');
            return false;
        }
        $this->logger->log('module enabled');


        /** @var Mage_Shipping_Model_Rate_Result $result */
        $result = Mage::getModel('shipping/rate_result');

        $destCountry = $request->getDestCountryId();
        $destRegion = $request->getDestRegionCode();

        $destRegionId = $request->getDestRegionId();

        $destPostCode = $request->getDestPostcode();

        $this->logger->log("Destination: COUNTRY: " . $destCountry . " - REGION ID: " . $destRegionId . " - REGION CODE: " . $destRegion . " - POSTCODE: " . $destPostCode);


        $shipmentConfigurationMode = $helper->getShipmentConfigurationMode();


        $shipments = array();
        $baseSubtotalInclTax = $request->getBaseSubtotalInclTax();

        $this->logger->log("SHIPMENTCONFIGURATIONMODE: " . $shipmentConfigurationMode);

        if ($shipmentConfigurationMode == self::SHIPMENT_CONFIGURATION_MODE_ONE_SHIPMENT_PER_ITEM)
        {
            /** @var $item Mage_Sales_Model_Quote_Item */
            $iteration = 1;
            foreach ($request->getAllItems() as $item)
            {
                for ($i = 1; $i <= $item->getQty(); $i++)
                {
                    $this->logger->log("Product Iteration: " . $iteration);
                    $weight = $item->getWeight();
                    $shipments = $this->getRates($destCountry, $destRegion, $destPostCode, $baseSubtotalInclTax, $weight, 1, $shipments, $iteration);
                    $iteration++;
                }
            }
        }
        elseif ($shipmentConfigurationMode == self::SHIPMENT_CONFIGURATION_MODE_ONE_SHIPMENT_PER_SHOPPING_CART_SINGLE_PARCEL)
        {
            $maxPackageWeight = $helper->getMaxPackageWeight();
            $boxesWeights = array();

            foreach ($request->getAllItems() as $item)
            {
                for ($i = 1; $i <= $item->getQty(); $i++)
                {
                    $canAddToExistingBox = false;
                    for ($j = 0; $j <= count($boxesWeights); $j++)
                    {
                        $newWeight = $boxesWeights[$j] + $item->getWeight();
                        if ($newWeight < $maxPackageWeight)
                        {
                            $canAddToExistingBox = true;
                            $boxesWeights[$j] = $newWeight;
                            break;
                        }
                    }
                    if (!$canAddToExistingBox)
                    {
                        $boxesWeights[] = $item->getWeight();
                    }
                }
            }

            $numBoxes = count($boxesWeights);

            $this->logger->log("Num Boxes: " . $numBoxes);

            $shipments = $this->getRates($destCountry, $destRegion, $destPostCode, $baseSubtotalInclTax, $boxesWeights, $numBoxes);
        }
        elseif ($shipmentConfigurationMode == self::SHIPMENT_CONFIGURATION_MODE_ONE_SHIPMENT_PER_SHOPPING_CART_MULTI_PARCEL)
        {
            $boxesWeights = array();
            $numBoxes = 0;

            foreach ($request->getAllItems() as $item)
            {
                $numBoxes += $item->getQty();
                for ($i = 1; $i <= $item->getQty(); $i++)
                {
                    $boxesWeights[] = $item->getWeight();
                }
            }
            $this->logger->log("Num Boxes: " . $numBoxes);
            $shipments = $this->getRates($destCountry, $destRegion, $destPostCode, $baseSubtotalInclTax, $boxesWeights, $numBoxes);

        }


        //TODO- remove subzone if it is the same for all shipping methods

        $subZones = array();
        foreach ($shipments as $shipment)
        {

            if (!in_array($shipment->subzone_id, $subZones))
            {
                array_push($subZones, $shipment->subzone_id);
            }

        }
        $useSubZone = false;
        if (count($subZones) > 1)
        {
            $useSubZone = true;
        }


        if (empty($shipments))
        {
            $errorTitle = 'Unable to retrieve shipping methods';
            $error = Mage::getModel('shipping/rate_result_error');
            $error->setCarrier($this->_code);
            $error->setCarrierTitle($this->getConfigData('title'));
            $error->setErrorMessage($errorTitle);
            $error->setErrorMessage($this->getConfigData('specificerrmsg'));
            $result->append($error);
        }
        else
        {

            foreach ($shipments as $shipment)
            {
                if ($useSubZone)
                {
                    $currentRate = $this->_getRate($shipment->title_full, $shipment->shipment_code, $shipment->price);
                }
                else
                {
                    $currentRate = $this->_getRate($shipment->title, $shipment->shipment_code, $shipment->price);
                }

                $result->append($currentRate);
            }
        }

        return $result;
    }


    private function getRates($destCountry, $destRegion, $destPostCode, $baseSubtotalInclTax, $weight, $boxes, $oldResults = array(), $iteration = 1)
    {
        $this->logger->log("getRates");

        /** @var $helper Mbe_Shipping_Helper_Data */
        $helper = Mage::helper('mbe_shipping');

        /** @var $ws Mbe_Shipping_Model_Ws */
        $ws = Mage::getModel('mbe_shipping/ws');

        $result = array();
        $newResults = array();


        $shipments = $ws->estimateShipping($destCountry, $destRegion, $destPostCode, $weight, $boxes);

        $this->logger->logVar($shipments, 'ws estimateShipping result');

        if ($shipments)
        {

            $allowedShipmentServicesArray = $helper->getAllowedShipmentServicesArray();

            foreach ($shipments as $shipment)
            {

                //TODO: check if is sufficient use service
                $shipmentMethod = $shipment->Service;

                $shipmentMethodKey = $shipment->Service . "_" . $shipment->IdSubzone;

                if (in_array($shipmentMethod, $allowedShipmentServicesArray))
                {
                    $shipmentTitle = $helper->__($shipment->ServiceDesc);
                    $shipmentTitle .= " - " . $helper->__($shipment->SubzoneDesc);

                    $shipmentPrice = $shipment->NetShipmentTotalPrice;

                    $shipmentPrice = $this->applyFee($shipmentPrice, $boxes);


                    $shippingThreshold = $helper->getThresholdByShippingServrice($shipmentMethod);
                    //var_dump($shippingThreshold);
                    if ($shippingThreshold != null && $baseSubtotalInclTax >= $shippingThreshold)
                    {
                        $shipmentPrice = 0;
                    }


                    //TODO: check if is necessary to save in some mode the courier data


                    $current = new stdClass();
                    $current->title = $helper->__($shipment->ServiceDesc);
                    $current->title_full = $shipmentTitle;
                    $current->method = $shipmentMethod;
                    $current->price = $shipmentPrice;
                    $current->subzone = $shipment->SubzoneDesc;
                    $current->subzone_id = $shipment->IdSubzone;
                    $current->shipment_code = $shipmentMethodKey;


                    $newResults[$shipmentMethodKey] = $current;
                    /*
                    if (array_key_exists($shipmentMethod, $oldResult)) {
                        $current->price = $shipmentPrice + $oldResult[$shipmentMethod]->price;
                    }

                    if (array_key_exists($shipmentMethod, $oldResult)) {

                        $result[$shipmentMethod]->price += $shipmentPrice;

                    }
                    else{
                        if($iteration == 1){
                            $current->price = $shipmentPrice;
                            if($iteration == 1)
                                $current = new stdClass();
                            $current->title = $shipmentTitle;
                            $current->method = $shipmentMethod;
                            $current->price = $shipmentPrice;
                            $result[$shipmentMethod] = $current;
                        }
                    }
                    //$currentRate = $this->_getRate($shipmentTitle, $shipmentMethod, $shipmentPrice);
                    */
                }
            }
            if ($iteration == 1)
            {
                $result = $newResults;
            }
            else
            {

                foreach ($newResults as $newResultKey => $newResult)
                {
                    if (array_key_exists($newResultKey, $oldResults))
                    {
                        $newResult->price += $oldResults[$newResultKey]->price;
                        $result[$newResultKey] = $newResult;
                    }

                }
            }

        }

        return $result;
    }

    /**
     * Returns Allowed shipping methods
     *
     * @return array
     */
    public function getAllowedMethods()
    {
        return array(
            'standard' => 'Standard delivery',
            'express' => 'Express delivery',
        );
    }

    //
    /**
     * Get rate object based on name and price
     *
     * @param $title string Title name
     * @param $method string Method name
     * @param $price float Shipping Method price
     * @return Mage_Shipping_Model_Rate_Result_Method
     */
    protected function _getRate($title, $method, $price)
    {
        /** @var $helper Mbe_Shipping_Helper_Data */
        $helper = Mage::helper('mbe_shipping');
        $price = $helper->round($price);

        /** @var $rate Mage_Shipping_Model_Rate_Result_Method */
        $rate = Mage::getModel('shipping/rate_result_method');
        $rate->setCarrier($this->_code);
        $rate->setCarrierTitle($helper->__($this->getConfigData('title')));
        $rate->setMethod($method);
        $rate->setMethodTitle($title);
        $rate->setPrice($price);
        $rate->setCost(0);
        return $rate;
    }


    public function applyFee($value, $packages = 1)
    {
        $handlingType = $this->getConfigData("handling_type");
        $handlingAction = $this->getConfigData("handling_action");
        $handlingFee = $this->getConfigData("handling_fee");

        if ($handlingAction == self::HANDLING_TYPE_PER_SHIPMENT)
        {
            $packages = 1;
        }

        if (self::HANDLING_TYPE_FIXED == $handlingType)
        {
            //fixed
            $result = $value + $handlingFee * $packages;
        }
        else
        {
            //percent
            $result = $value * (100 + $handlingFee) / 100;
        }

        return $result;

    }

    public function isTrackingAvailable()
    {
        return true;
    }

    public function getTrackingInfo($tracking)
    {

        $helper = Mage::helper('mbe_shipping');
        $trackingHelper = Mage::helper('mbe_shipping/tracking');
        $mbeUrl = $trackingHelper->getTrackingUrlBySystem($helper->getCountry());

        $track = Mage::getModel('shipping/tracking_result_status');


        $track->setUrl($mbeUrl . $tracking)
            ->setTracking($tracking)
            ->setCarrierTitle($this->getConfigData('title'));
        return $track;
    }
}