<?php

class Mbe_Shipping_Model_Observer
{

    /**
     * Check if order need to create mbe shipping
     *
     * @param Varien_Event_Observer $observer
     * @return $this
     */
    public function checkOrder(Varien_Event_Observer $observer)
    {
        /**@var $helper Mbe_Shipping_Helper_Data */
        $helper = Mage::helper('mbe_shipping');
        if ($helper->isEnabled())
        {
            $order = $observer->getOrder();
            //only if order is shipped with mbe
            if ($helper->isMbeShipping($order))
            {
                $stateProcessing = Mage_Sales_Model_Order::STATE_PROCESSING;
                if ($order->getState() == $stateProcessing && $order->getOrigData('state') != $stateProcessing)
                {
                    $this->createShipment($order);
                }

                $stateNew = Mage_Sales_Model_Order::STATE_NEW;
                //$statePending = Mage_Sales_Model_Order::STATE_PENDING_PAYMENT;
                $paymentMethod = $order->getPayment()->getMethodInstance()->getCode();
                if ($order->getState() == $stateNew)
                {
                    if ($paymentMethod == "checkmo")
                    {
                        $this->createShipment($order);
                    }
                }
            }
        }
        return $this;
    }

    //http://www.amitbera.com/programmatically-create-shipment-of-a-new-order-in-magento/
    //http://excellencemagentoblog.com/blog/2012/06/26/useful-code-snippets/
    //http://www.magebuzz.com/blog/magento-create-multi-shipment-for-an-order-programmatically/
    //http://magento.stackexchange.com/questions/3252/programmatically-creating-shipments
    //http://stackoverflow.com/questions/16812397/programatically-create-shipment-for-selected-items-qty-on-order-in-magento
    private function createShipment(Mage_Sales_Model_Order $o)
    {
        /** @var  $logger Mbe_Shipping_Helper_Logger */
        $logger = Mage::helper('mbe_shipping/logger');
        /**@var $helper Mbe_Shipping_Helper_Data */
        $helper = Mage::helper('mbe_shipping');

        $logger->log("CREATESHIPMENT");

        $order = Mage::getModel('sales/order')->load($o->getId());


        if ($order->canShip())
        {

            $shipmentConfigurationMode = $helper->getShipmentConfigurationMode();

            if ($shipmentConfigurationMode == Mbe_Shipping_Model_Carrier::SHIPMENT_CONFIGURATION_MODE_ONE_SHIPMENT_PER_ITEM)
            {

                foreach ($order->getAllItems() as $eachOrderItem)
                {
                    $itemQty = $eachOrderItem->getQtyOrdered()
                        - $eachOrderItem->getQtyShipped()
                        - $eachOrderItem->getQtyRefunded()
                        - $eachOrderItem->getQtyCanceled();

                    $itemWeight = $eachOrderItem->getWeight();
                    for ($i = 1; $i <= $itemQty; $i++)
                    {
                        $qty = array();
                        $qty[$eachOrderItem->getId()] = 1;

                        $logger->logVar($qty, 'qty');
                        $this->createSingleShipment($order, $itemWeight, 1, $qty);
                    }
                }
            }
            elseif ($shipmentConfigurationMode == Mbe_Shipping_Model_Carrier::SHIPMENT_CONFIGURATION_MODE_ONE_SHIPMENT_PER_SHOPPING_CART_SINGLE_PARCEL)
            {
                $maxPackageWeight = $helper->getMaxPackageWeight();
                $boxesWeights = array();

                foreach ($order->getAllItems() as $item)
                {
                    for ($i = 1; $i <= $item->getQtyOrdered(); $i++)
                    {
                        $canAddToExistingBox = false;
                        for ($j = 0; $j <= count($boxesWeights); $j++)
                        {
                            $newWeight = $boxesWeights[$j] + $item->getWeight();
                            if ($newWeight < $maxPackageWeight)
                            {
                                $canAddToExistingBox = true;
                                $boxesWeights[$j] = $newWeight;
                                break;
                            }
                        }
                        if (!$canAddToExistingBox)
                        {
                            $boxesWeights[] = $item->getWeight();
                        }
                    }
                }
                $numBoxes = count($boxesWeights);
                $logger->logVar($numBoxes, "boxes amount");
                $logger->logVar($boxesWeights, "boxes weights");
                $this->createSingleShipment($order, $boxesWeights, $numBoxes);
            }
            elseif ($shipmentConfigurationMode == Mbe_Shipping_Model_Carrier::SHIPMENT_CONFIGURATION_MODE_ONE_SHIPMENT_PER_SHOPPING_CART_MULTI_PARCEL)
            {
                $boxesWeights = array();
                $numBoxes = 0;

                foreach ($order->getAllItems() as $item)
                {
                    $numBoxes += $item->getQtyOrdered();
                    for ($i = 1; $i <= $item->getQtyOrdered(); $i++)
                    {
                        $boxesWeights[] = $item->getWeight();
                    }
                }
                $logger->logVar($numBoxes, "boxes amount");
                $logger->logVar($boxesWeights, "boxes weights");
                $this->createSingleShipment($order, $boxesWeights, $numBoxes);
            }

        }

    }

    public function createSingleShipment($order, $weight, $boxes, $qty = array())
    {
        $sendEmail = true;
        $includeComment = true;
        $comment = "Shipment created through MBE";

        /**@var $helper Mbe_Shipping_Helper_Data */
        $helper = Mage::helper('mbe_shipping');
        /** @var  $logger Mbe_Shipping_Helper_Logger */
        $logger = Mage::helper('mbe_shipping/logger');


        /* prepare to create shipment */
        /* @var $shipment Mage_Sales_Model_Order_Shipment */
        $shipment = $order->prepareShipment($qty);
        if ($shipment)
        {
            $shipment->register();
            $shipment->addComment($comment, $sendEmail && $includeComment);
            $shipment->getOrder()->setIsInProcess(true);
            try
            {
                $transactionSave = Mage::getModel('core/resource_transaction')
                    ->addObject($shipment)
                    ->addObject($shipment->getOrder())
                    ->save();
                //TODO: create MBE SHIPMENT


                $fileName = $shipment->getIncrementId();


                /** @var  $ws Mbe_Shipping_Model_Ws */
                $ws = Mage::getModel('mbe_shipping/ws');


                $shippingMethod = $order->getShippingMethod();

                $service = $helper->getShippingCodeFromMagentoShippingMethod($shippingMethod);
                $subzone = $helper->getShippingSubZoneFromMagentoShippingMethod($shippingMethod);

                $shippingAddress = $order->getShippingAddress();
                $firstName = $shippingAddress->getFirstname();
                $lastName = $shippingAddress->getLastname();
                $addressArray = $shippingAddress->getStreet();
                $address = implode($addressArray, " ");
                $phone = $shippingAddress->getTelephone();
                $city = $shippingAddress->getCity();
                $countryId = $shippingAddress->getCountryId();
                /*
                $region = $shippingAddress->getRegionId();
                if (!$region) {
                    $region = $shippingAddress->getRegion();
                }
                */
                $region = $shippingAddress->getRegion();
                $postCode = $shippingAddress->getPostcode();
                $email = $shippingAddress->getEmail();

                $mbeShipment = $ws->createShipping($countryId, $region, $postCode, $weight, $boxes, $service, $subzone, $firstName, $lastName, $address, $phone, $city, $email);
                $logger->logVar($mbeShipment, "MBE SHIPMENT");

                $trackingNumber = $mbeShipment->MasterTrackingMBE;
                $label = $mbeShipment->Labels->Label;
                $this->saveShipmentDocument($label->Type, $label->Stream, $fileName);


                $track = Mage::getModel('sales/order_shipment_track')
                    ->setShipment($shipment)
                    ->setData('title', 'Mbe Shipping')
                    ->setData('number', $trackingNumber)
                    ->setData('carrier_code', 'mbe_shipping')
                    ->setData('order_id', $shipment->getData('order_id'))
                    ->save();

                $shipment->sendEmail($sendEmail, ($includeComment ? $comment : ''));
            }
            catch (Mage_Core_Exception $e)
            {
                $logger->log($e->getMessage());
            }

        }
    }

    /**
     * Method called from magento cron based on module configuration
     */
    function executeCron()
    {
        /**@var $helper Mbe_Shipping_Helper_Logger */
        $logger = Mage::helper('mbe_shipping/logger');
        /**@var $helper Mbe_Shipping_Helper_Data */
        $helper = Mage::helper('mbe_shipping');

        $time = Mage::getModel('core/date')->date('H:i');
        $logger->log("Cron Execution at " . $time);
        if ($helper->isEnabled())
        {
            $this->closeShipments();
        }
        else
        {
            $logger->log("Cron Execution stopped because module is disabled or user not active");
        }
    }

    /**
     * Method that close all not closed shipments
     */
    function closeShipments()
    {
        /**@var $helper Mbe_Shipping_Helper_Data */
        $helper = Mage::helper('mbe_shipping');
        /**@var $logger Mbe_Shipping_Helper_Logger */
        $logger = Mage::helper('mbe_shipping/logger');
        /** @var  $ws Mbe_Shipping_Model_Ws */
        $ws = Mage::getModel('mbe_shipping/ws');


        if ($ws->mustCloseShipments())
        {
            if ($helper->getShipmentsClosureMode() == Mbe_Shipping_Helper_Data::MBE_CLOSURE_MODE_AUTOMATICALLY)
            {

                $time = time();
                $to = date('Y-m-d H:i:s', $time);
                //$lastTime = $time - 172800; // 60*60*24*2
                //TODO: check if decrease this value
                $lastTime = $time - 60 * 60 * 24 * 30; // 60*60*24*2
                $from = date('Y-m-d H:i:s', $lastTime);

                $orderCollection = Mage::getResourceModel('sales/order_collection');
                $stateComplete = Mage_Sales_Model_Order::STATE_COMPLETE;
                $orderCollection
                    ->addFieldToFilter('state', $stateComplete)
                    ->addAttributeToSelect('entity_id')
                    //->addAttributeToSelect('increment_id')
                    ->addAttributeToSelect('created_at')
                    ->addAttributeToFilter('created_at', array('from' => $from, 'to' => $to))
                    ->load();


                foreach ($orderCollection as $o)
                {

                    $order = Mage::getModel('sales/order')->load($o->getId());
                    $orderIncrementId = $order->getIncrementId();

                    if ($helper->isMbeShipping($order))
                    {

                        //TODO: get shipments and close theme if theme are opened

                        foreach ($order->getShipmentsCollection() as $shipment)
                        {
                            $shipmentIncrementId = $shipment->getIncrementId();

                            $tracks = $shipment->getAllTracks();
                            foreach ($tracks as $track)
                            {
                                $trackingNumber = $track->getTrackNumber();

                                if ($helper->isTrackingOpen($trackingNumber))
                                {
                                    $msg = "Order: " . $orderIncrementId . " - Shipment: " . $shipmentIncrementId . " is open.";
                                    $logger->log($msg);

                                    $ws->closeTrackingNumber($trackingNumber);
                                }
                            }
                        }

                    }
                }

            }
        }
    }


    /** Method that save shipment document
     * @param $type String document type
     * @param $content String document content
     * @param $filename String filename for new document to save
     */
    public function saveShipmentDocument($type, $content, $filename)
    {
        /**@var $helper Mbe_Shipping_Helper_Data */
        $helper = Mage::helper('mbe_shipping');

        /**@var $logger Mbe_Shipping_Helper_Logger */
        $logger = Mage::helper('mbe_shipping/logger');

        $ext = "txt";
        if ($type == "HTML")
        {
            $ext = "html";
        }
        elseif ($type == "PDF")
        {
            $ext = "pdf";
        }
        $filePath = $helper->getShipmentFilePath($filename, $ext);
        $saveResult = file_put_contents($filePath, $content);

        $message = "Saving shipping document :" . $filePath;

        if ($saveResult)
        {
            $message .= " OK";
        }
        else
        {
            $message .= " FAILURE";
        }

        $logger->log($message);
    }
}